# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-07 17:20
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('raw', models.CharField(max_length=300)),
                ('house', models.CharField(max_length=20)),
                ('latitude', models.DecimalField(decimal_places=6, max_digits=9)),
                ('longitude', models.DecimalField(decimal_places=6, max_digits=9)),
            ],
            options={
                'verbose_name_plural': 'Addresses',
            },
        ),
        migrations.CreateModel(
            name='AdministrativeArea',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name_plural': 'Countries',
            },
        ),
        migrations.CreateModel(
            name='Locality',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('admin_area', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='yamaps.AdministrativeArea')),
            ],
            options={
                'verbose_name_plural': 'Localities',
            },
        ),
        migrations.CreateModel(
            name='Street',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('locality', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='yamaps.Locality')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='administrativearea',
            name='country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='yamaps.Country'),
        ),
        migrations.AddField(
            model_name='address',
            name='street',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='yamaps.Street'),
        ),
    ]



from decimal import Decimal

from django import forms
from django.conf import settings
from django.utils.safestring import mark_safe

from .models import Address


def get_address_js():
    lang = "ru-RU"
    if settings.LANGUAGE_CODE:
        lang = settings.LANGUAGE_CODE
    yamaps_js_url = \
        "https://api-maps.yandex.ru/2.1/?lang={}".format(lang)
    return yamaps_js_url, "yamaps/js/yamaps-widget.js"


class AddressWidget(forms.TextInput):

    parts = ["latitude", "longitude", "country",
             "admin_area", "locality", "street", "house"]

    def __init__(self, *args, **kwargs):
        attrs = kwargs.get('attrs', {})
        classes = attrs.get('class', '')
        classes += (' ' if classes else '') + 'yamaps'
        attrs['class'] = classes
        kwargs['attrs'] = attrs
        super(AddressWidget, self).__init__(*args, **kwargs)

    def _media(self):
        return forms.Media(js=get_address_js())

    media = property(_media)

    def render(self, name, value, attrs=None, **kwargs):

        addr = {}
        if isinstance(value, Address):
            addr = value.as_dict()
        elif isinstance(value, int):
            addr = Address.objects.get(pk=value).as_dict()

        # Raw address input will be readonly.
        if "readonly" not in attrs:
            attrs["readonly"] = "readonly"

        elems = [super(AddressWidget, self)
                 .render(name, addr.get("raw", ""), attrs, **kwargs)]

        # Hidden address parts.
        elems.append('<div id="{}_components">'.format(name))

        for part in self.parts:
            elems.append(
                '<input type="hidden" name="{}_{}" value="{}" />'.format(
                    name, part, addr.get(part, "")
                ))

        elems.append("</div>")

        # Yandex map.
        elems.append('<div id="{}_map" style="width: 700px; height: 500px;"></div>'
                     .format(name))

        return mark_safe('\n'.join(elems))

    def value_from_datadict(self, data, files, name):
        raw = data.get(name, None)
        if not raw:
            return raw

        addr = {p: data.get("{}_{}".format(name, p)) for p in self.parts}
        addr["raw"] = raw
        return addr


class AddressField(forms.Field):
    widget = AddressWidget

    def to_python(self, value):

        if value is None or value == "":
            return None

        for field in ["latitude", "longitude"]:
            if field in value:
                if value[field]:
                    try:
                        value[field] = Decimal(value[field])
                    except:
                        raise forms.ValidationError(
                            "Invalid value for %(field)s",
                            code="invalid",
                            params={"field": field}
                        )
                else:
                    raise forms.ValidationError(
                        "Field %(field)s is required",
                        code="invalid",
                        params={"field": field}
                    )

        return Address.objects.create_from_dict(value)



from django.contrib import admin

from .models import Country
from .models import AdministrativeArea
from .models import Locality
from .models import Street
from .models import Address


admin.site.register(Country)
admin.site.register(AdministrativeArea)
admin.site.register(Locality)
admin.site.register(Street)
admin.site.register(Address)



from django.db import models


class AddressPart(models.Model):
    name = models.CharField(max_length=50)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class Country(AddressPart):
    pass

    class Meta:
        verbose_name_plural = "Countries"


class AdministrativeArea(AddressPart):
    country = models.ForeignKey(Country)

    def __str__(self):
        return "{}, {}".format(str(self.country), self.name)


class Locality(AddressPart):
    admin_area = models.ForeignKey(AdministrativeArea)

    def __str__(self):
        return "{}, {}".format(str(self.admin_area), self.name)

    class Meta:
        verbose_name_plural = "Localities"


class Street(AddressPart):
    locality = models.ForeignKey(Locality)

    def __str__(self):
        return "{}, {}".format(str(self.locality), self.name)


class AddressManager(models.Manager):

    def create_from_dict(self, data):
        country, _ = Country.objects.get_or_create(name=data.get("country"))

        admin_area, _ = AdministrativeArea\
            .objects\
            .get_or_create(
                name=data.get("admin_area"),
                country=country
            )

        locality, _ = Locality\
            .objects\
            .get_or_create(
                name=data.get("locality"),
                admin_area=admin_area
            )

        street, _ = Street.objects.get_or_create(
            name=data.get("street"),
            locality=locality
        )
        addr, _ = self.get_or_create(
            raw=data.get("raw"),
            street=street,
            house=data.get("house"),
            latitude=data.get("latitude"),
            longitude=data.get("longitude")
        )
        return addr


class Address(models.Model):
    raw = models.CharField(max_length=300)
    street = models.ForeignKey(Street)
    house = models.CharField(max_length=20)
    latitude = models.DecimalField(max_digits=9, decimal_places=6)
    longitude = models.DecimalField(max_digits=9, decimal_places=6)

    objects = AddressManager()

    def as_dict(self):

        return {
            "raw": self.raw,
            "latitude": self.latitude,
            "longitude": self.longitude,
            "country": self.street.locality.admin_area.country.name,
            "admin_area": self.street.locality.admin_area.name,
            "locality": self.street.locality.name,
            "street": self.street.name,
            "house": self.house,
        }

    class Meta:
        verbose_name_plural = "Addresses"

    def __str__(self):
        return self.raw
